@@ -1,42 +1,74 @@
 from __future__ import annotations
+
 import asyncio
-from contextlib import asynccontextmanager
-from contextvars import ContextVar
-from datetime import datetime
-from enum import Flag, auto
-from functools import reduce
-from inspect import Parameter, Signature, iscoroutinefunction
+import contextlib
+import contextvars
+import datetime
+import enum
+import functools
 import inspect
-from typing import Any, AsyncContextManager, Awaitable, Callable, Dict, Optional, Sequence, Type, TypeVar, List, Union
-from weakref import WeakKeyDictionary
+import sys
+import weakref
+from typing import Any, AsyncContextManager, Awaitable, Callable, Dict, List, Optional, Sequence, Type, TypeVar, Union
 
-from pydantic import BaseModel, validator
-from pydantic.fields import ModelField
-from pydantic.main import ModelMetaclass
-from servo.utilities import join_to_series
+import pydantic
+import pydantic.main
+
+import servo.utilities.inspect
+import servo.utilities.strings
+
+__all__ = [
+    "Event",
+    "EventHandler",
+    "EventResult",
+    "Preposition",
+    "create_event",
+    "event",
+    "before_event",
+    "on_event",
+    "after_event",
+    "event_handler",
+]
+
+_signature_cache: Dict[str, inspect.Signature] = {}
 
 
-_signature_cache: Dict[str, Signature] = {}
-
-class Event(BaseModel):
+class Event(pydantic.BaseModel):
     """
     The Event class defines a named event that can be dispatched and
     processed with before, on, and after handlers.
     """
-    name: str
-    on_handler_context_manager: Callable[[None], AsyncContextManager]
 
-    def __init__(self, name: str, signature: Signature, *args, **kwargs):
+    name: str
+    """Unique name of the event.
+    """
+
+    module: Optional[str] = None
+    """Module that defined the event.
+    """
+
+    on_handler_context_manager: Callable[[None], AsyncContextManager]
+    """Context manager callable providing a default on event handler for the event.
+    """
+
+    def __init__(
+        self, name: str, signature: inspect.Signature, *args, **kwargs
+    ) -> None:
         _signature_cache[name] = signature
         super().__init__(name=name, *args, **kwargs)
 
     @property
-    def signature(self) -> Signature:
+    def signature(self) -> inspect.Signature:
         # Hide signature from Pydantic
         return _signature_cache[self.name]
 
     def __hash__(self):
-        return hash((self.name, self.signature,))
+        return hash(
+            (
+                self.name,
+                self.signature,
+            )
+        )
 
     def __str__(self):
         return self.name
@@ -51,14 +83,14 @@ class Event(BaseModel):
     def dict(
         self,
         *,
-        include: Union['AbstractSetIntStr', 'MappingIntStrAny'] = None,
-        exclude: Union['AbstractSetIntStr', 'MappingIntStrAny'] = None,
+        include: Union[pydantic.AbstractSetIntStr, pydantic.MappingIntStrAny] = None,
+        exclude: Union[pydantic.AbstractSetIntStr, pydantic.MappingIntStrAny] = None,
         by_alias: bool = False,
         skip_defaults: bool = None,
         exclude_unset: bool = False,
         exclude_defaults: bool = False,
         exclude_none: bool = False,
-    ) -> 'DictStrAny':
+    ) -> pydantic.DictStrAny:
         if exclude is None:
             exclude = set()
         exclude.add("on_handler_context_manager")
@@ -69,7 +101,7 @@ class Event(BaseModel):
             skip_defaults=skip_defaults,
             exclude_unset=exclude_unset,
             exclude_defaults=exclude_defaults,
-            exclude_none=exclude_none
+            exclude_none=exclude_none,
         )
 
     class Config:
@@ -79,13 +111,12 @@ class Event(BaseModel):
 EventCallable = TypeVar("EventCallable", bound=Callable[..., Any])
 
 
-class Preposition(Flag):
-    BEFORE = auto()
-    ON = auto()
-    AFTER = auto()
+class Preposition(enum.Flag):
+    BEFORE = enum.auto()
+    ON = enum.auto()
+    AFTER = enum.auto()
     ALL = BEFORE | ON | AFTER
 
-
     @classmethod
     def from_str(cls, prep: str) -> "Preposition":
         if not isinstance(prep, str):
@@ -109,18 +140,15 @@ class Preposition(Flag):
             return "after"
 
 
-class EventContext(BaseModel):
+class EventContext(pydantic.BaseModel):
     event: Event
     preposition: Preposition
-    created_at: datetime = None
+    created_at: datetime.datetime = None
 
-    @classmethod # Usable as a validator
-    def from_str(cls, event_str) -> Optional['EventContext']:
+    @classmethod  # Usable as a validator
+    def from_str(cls, event_str) -> Optional["EventContext"]:
         if event := get_event(event_str, None):
-            return EventContext(
-                preposition=Preposition.ON,
-                event=event
-            )
+            return EventContext(preposition=Preposition.ON, event=event)
 
         components = event_str.split(":", 1)
         if len(components) < 2:
@@ -136,16 +164,12 @@ class EventContext(BaseModel):
         if not event:
             return None
 
-        return EventContext(
-            preposition=Preposition.from_str(preposition),
-            event=event
-        )
+        return EventContext(preposition=Preposition.from_str(preposition), event=event)
 
-
-    @validator("created_at", pre=True, always=True)
+    @pydantic.validator("created_at", pre=True, always=True)
     @classmethod
     def set_created_at_now(cls, v):
-        return v or datetime.now()
+        return v or datetime.datetime.now()
 
     def is_before(self) -> bool:
         return self.preposition == Preposition.BEFORE
@@ -183,7 +207,8 @@ class EventContext(BaseModel):
 def validate_event_contexts(
     cls,
     value: Union[str, EventContext, Sequence[Union[str, EventContext]]],
-    field: ModelField) -> Union[str, List[str]]:
+    field: pydantic.ModelField,
+) -> Union[str, List[str]]:
     """
     A Pydantic validator function that ensures that the input value or values are
     valid event context identifiers (e.g. "measure", "before:adjust", etc)
@@ -202,18 +227,18 @@ def validate_event_contexts(
         raise ValueError(f"Invalid value for {field.name}")
 
 
-class EventHandler(BaseModel):
+class EventHandler(pydantic.BaseModel):
     event: Event
     preposition: Preposition
     kwargs: Dict[str, Any]
-    connector_type: Optional[Type['BaseConnector']] # NOTE: Optional due to decorator
+    connector_type: Optional[Type["servo.connector.BaseConnector"]]  # NOTE: Optional due to decorator
     handler: EventCallable
 
     def __str__(self):
         return f"{self.connector_type}({self.preposition}:{self.event}->{self.handler})"
 
 
-class EventResult(BaseModel):
+class EventResult(pydantic.BaseModel):
     """
     Encapsulates the result of a dispatched Connector event
     """
@@ -221,14 +246,14 @@ class EventResult(BaseModel):
     event: Event
     preposition: Preposition
     handler: EventHandler
-    connector: BaseConnector
-    created_at: datetime = None
+    connector: "servo.connector.BaseConnector"
+    created_at: datetime.datetime = None
     value: Any
 
-    @validator("created_at", pre=True, always=True)
+    @pydantic.validator("created_at", pre=True, always=True)
     @classmethod
     def set_created_at_now(cls, v):
-        return v or datetime.now()
+        return v or datetime.datetime.now()
 
 
 class EventError(RuntimeError):
@@ -264,10 +289,17 @@ def get_event(name: str, default=...) -> Optional[Event]:
 
 def create_event(
     name: str,
-    signature: Union[Callable[[Any], Awaitable], Signature],
+    signature: Union[Callable[[Any], Awaitable], inspect.Signature],
+    *,
+    module: Optional[str] = None,
 ) -> Event:
     """
     Create an event programmatically from a name and function signature.
+
+    Args:
+        name: The name of the event to be created.
+        signature: The method signature of on event handlers of the event.
+        module: The module that defined the event. When `None`, inferred via the `inspect` module.
     """
     if _events.get(name, None):
         raise ValueError(f"Event '{name}' has already been created")
@@ -277,16 +309,18 @@ def create_event(
         async def fn(self) -> None:
             yield
 
-        return asynccontextmanager(fn)
+        return contextlib.asynccontextmanager(fn)
 
     if callable(signature):
         if inspect.isasyncgenfunction(signature):
             # We have an async generator function defining setup/teardown activities, wrap into a context manager
             # This is useful for shared behaviors like startup delays, settlement times, etc.
-            on_handler_context_manager = asynccontextmanager(signature)
+            on_handler_context_manager = contextlib.asynccontextmanager(signature)
 
-        elif not iscoroutinefunction(signature):
-            raise ValueError(f"events must be async: add `async` prefix to your function declaration and await as necessary ({signature})")
+        elif not inspect.iscoroutinefunction(signature):
+            raise ValueError(
+                f"events must be async: add `async` prefix to your function declaration and await as necessary ({signature})"
+            )
 
         else:
             # Sanity check callables that don't yield are stubs
@@ -295,14 +329,19 @@ def create_event(
                 lines = inspect.getsourcelines(signature)
                 last = lines[0][-1]
                 if not last.strip() in ("pass", "..."):
-                    raise ValueError("function body of event declaration must be an async generator or a stub using `...` or `pass` keywords")
+                    raise ValueError(
+                        "function body of event declaration must be an async generator or a stub using `...` or `pass` keywords"
+                    )
 
                 # use the default since our input doesn't yield
                 on_handler_context_manager = _default_context_manager()
 
-            except OSError as error:
+            except OSError:
                 from servo.logging import logger
-                logger.warning(f"unable to inspect event declaration for '{name}': dropping event body and proceeding")
+
+                logger.warning(
+                    f"unable to inspect event declaration for '{name}': dropping event body and proceeding"
+                )
                 on_handler_context_manager = _default_context_manager()
 
     else:
@@ -311,12 +350,12 @@ def create_event(
 
     signature = (
         signature
-        if isinstance(signature, Signature)
-        else Signature.from_callable(signature)
+        if isinstance(signature, inspect.Signature)
+        else inspect.Signature.from_callable(signature)
     )
     if list(
         filter(
-            lambda param: param.kind == Parameter.VAR_POSITIONAL,
+            lambda param: param.kind == inspect.Parameter.VAR_POSITIONAL,
             signature.parameters.values(),
         )
     ):
@@ -324,7 +363,17 @@ def create_event(
             f"Invalid signature: events cannot declare variable positional arguments (e.g. *args)"
         )
 
-    event = Event(name=name, signature=signature, on_handler_context_manager=on_handler_context_manager)
+    # Get the module from the calling stack frame
+    if module is None:
+        localns = inspect.currentframe().f_back.f_locals
+        module = localns.get("__module__", None)
+
+    event = Event(
+        name=name,
+        signature=signature,
+        module=module,
+        on_handler_context_manager=on_handler_context_manager,
+    )
     _events[name] = event
     return event
 
@@ -343,13 +392,14 @@ def event(
 
     def decorator(fn: EventCallable) -> EventCallable:
         event_name = name if name else fn.__name__
+        module = inspect.currentframe().f_back.f_locals.get("__module__", None)
         if handler:
             # If the method body is a handler, pass the signature directly into `create_event`
             # as we are going to pass the method body into `on_event`
-            signature = Signature.from_callable(fn)
-            create_event(event_name, signature)
+            signature = inspect.Signature.from_callable(fn)
+            create_event(event_name, signature, module=module)
         else:
-            create_event(event_name, fn)
+            create_event(event_name, fn, module=module)
 
         if handler:
             decorator = on_event(event_name)
@@ -429,25 +479,71 @@ def event_handler(
             raise ValueError(f"Unknown event '{name}'")
         if preposition != Preposition.ON:
             name = f"{preposition}:{name}"
-        handler_signature = Signature.from_callable(fn)
+
+        # Build namespaces that can resolve names for the event definition and handler
+        event_globalns = (
+            sys.modules[event.module].__dict__.copy() if event.module else {}
+        )
+        event_globalns.update(globals())
+        handler_signature = inspect.Signature.from_callable(fn)
+        handler_globalns = inspect.currentframe().f_back.f_globals
+        handler_localns = inspect.currentframe().f_back.f_locals
+
+        handler_mod_name = handler_localns.get("__module__", None)
+        handler_module = sys.modules[handler_mod_name] if handler_mod_name else None
 
         if preposition == Preposition.BEFORE:
-            before_handler_signature = Signature.from_callable(__before_handler)
-            _validate_handler_signature(
-                handler_signature,
-                event_signature=before_handler_signature,
-                handler_name=name,
+            before_handler_signature = inspect.Signature.from_callable(__before_handler)
+            servo.utilities.inspect.assert_equal_callable_descriptors(
+                servo.utilities.inspect.CallableDescriptor(
+                    signature=before_handler_signature,
+                    module=event.module,
+                    globalns=event_globalns,
+                    localns=locals(),
+                ),
+                servo.utilities.inspect.CallableDescriptor(
+                    signature=handler_signature,
+                    module=handler_module,
+                    globalns=handler_globalns,
+                    localns=handler_localns,
+                ),
+                name=name,
+                method=True,
             )
         elif preposition == Preposition.ON:
-            _validate_handler_signature(
-                handler_signature, event_signature=event.signature, handler_name=name
+            servo.utilities.inspect.assert_equal_callable_descriptors(
+                servo.utilities.inspect.CallableDescriptor(
+                    signature=event.signature,
+                    module=event.module,
+                    globalns=event_globalns,
+                    localns=locals(),
+                ),
+                servo.utilities.inspect.CallableDescriptor(
+                    signature=handler_signature,
+                    module=handler_module,
+                    globalns=handler_globalns,
+                    localns=handler_localns,
+                ),
+                name=name,
+                method=True,
             )
         elif preposition == Preposition.AFTER:
-            after_handler_signature = Signature.from_callable(__after_handler)
-            _validate_handler_signature(
-                handler_signature,
-                event_signature=after_handler_signature,
-                handler_name=name,
+            after_handler_signature = inspect.Signature.from_callable(__after_handler)
+            servo.utilities.inspect.assert_equal_callable_descriptors(
+                servo.utilities.inspect.CallableDescriptor(
+                    signature=after_handler_signature,
+                    module=event.module,
+                    globalns=event_globalns,
+                    localns=locals(),
+                ),
+                servo.utilities.inspect.CallableDescriptor(
+                    signature=handler_signature,
+                    module=handler_module,
+                    globalns=handler_globalns,
+                    localns=handler_localns,
+                ),
+                name=name,
+                method=True,
             )
         else:
             assert "Undefined preposition value"
@@ -469,203 +565,23 @@ def __after_handler(self, results: List[EventResult]) -> None:
     pass
 
 
-def _validate_handler_signature(
-    handler_signature: Signature, *, event_signature: Signature, handler_name: str
-) -> None:
-    """
-    Validates that the given handler signature is compatible with the event signature. Validation
-    checks the parameter and return value types using annotations. The intent is to immediately
-    expose errors in event handlers rather than encountering them at runtime (which may take
-    an arbitrary amount of time to trigger a given event). Raises a TypeError when an incompatibility
-    is encountered.
-
-    :param handler_signature: The event handler signature to validate.
-    :param event_signature: The reference event signature to validate against.
-    :param handler_name: The name of the handler for inclusion in error messages & logs.
-    """
-
-    # Skip the work if the signatures are identical
-    if handler_signature == event_signature:
-        return
-
-    handler_parameters: Mapping[str, Parameter] = handler_signature.parameters
-    handler_positional_parameters = list(
-        filter(
-            lambda param: param.kind
-            in [Parameter.POSITIONAL_ONLY, Parameter.VAR_POSITIONAL],
-            handler_parameters.values(),
-        )
-    )
-    handler_keyword_parameters = dict(
-        filter(
-            lambda item: item[1].kind
-            in [
-                Parameter.KEYWORD_ONLY,
-                Parameter.POSITIONAL_OR_KEYWORD,
-                Parameter.VAR_KEYWORD,
-            ],
-            handler_parameters.items(),
-        )
-    )
-
-    event_parameters: Mapping[str, Parameter] = event_signature.parameters
-    event_positional_parameters = list(
-        filter(
-            lambda param: param.kind
-            in [Parameter.POSITIONAL_ONLY, Parameter.VAR_POSITIONAL],
-            event_parameters.values(),
-        )
-    )
-    event_keyword_parameters = dict(
-        filter(
-            lambda item: item[1].kind
-            in [
-                Parameter.KEYWORD_ONLY,
-                Parameter.POSITIONAL_OR_KEYWORD,
-                Parameter.VAR_KEYWORD,
-            ],
-            event_parameters.items(),
-        )
-    )
-
-    # We assume instance methods
-    args = list(handler_parameters.keys())
-    first_arg = args.pop(0) if args else None
-    if first_arg != "self":
-        raise TypeError(
-            f"Invalid signature for '{handler_name}' event handler: {handler_signature}, \"self\" must be the first argument"
-        )
-
-    # Check return type annotation
-    # TODO: Handle various import formats on return annotation
-    if handler_signature.return_annotation != event_signature.return_annotation:
-        error_message = f"Invalid return type annotation for '{handler_name}' event handler: expected {event_signature.return_annotation}, but found {handler_signature.return_annotation}"
-        if isinstance(event_signature.return_annotation, str) and not isinstance(handler_signature.return_annotation, str):
-            error_message += "\nThe type annotations captured from the module defining the event handler are not string encoded. Add `from __future__ import annotations` to the top of the module implementation."
-        raise TypeError(error_message)
-
-    # Check for extraneous positional parameters on the handler
-    handler_positional_only = list(
-        filter(
-            lambda param: param.kind == Parameter.POSITIONAL_ONLY,
-            handler_positional_parameters,
-        )
-    )
-    event_positional_only = list(
-        filter(
-            lambda param: param.kind == Parameter.POSITIONAL_ONLY,
-            event_positional_parameters,
-        )
-    )
-    if len(handler_positional_only) > len(event_positional_only):
-        extra_param_names = sorted(
-            list(
-                set(map(lambda p: p.name, handler_positional_only))
-                - set(map(lambda p: p.name, event_positional_only))
-            )
-        )
-        raise TypeError(
-            f"Invalid type annotation for '{handler_name}' event handler: encountered extra positional parameters ({join_to_series(extra_param_names)})"
-        )
-
-    # Check for extraneous keyword parameters on the handler
-    handler_keyword_nonvar = dict(
-        filter(
-            lambda item: item[1].kind != Parameter.VAR_KEYWORD,
-            handler_keyword_parameters.items(),
-        )
-    )
-    event_keyword_nonvar = dict(
-        filter(
-            lambda item: item[1].kind != Parameter.VAR_KEYWORD,
-            event_keyword_parameters.items(),
-        )
-    )
-    extraneous_keywords = sorted(
-        list(set(handler_keyword_nonvar.keys()) - set(event_keyword_nonvar.keys()))
-    )
-    if extraneous_keywords:
-        raise TypeError(
-            f"Invalid type annotation for '{handler_name}' event handler: encountered extra parameters ({join_to_series(extraneous_keywords)})"
-        )
-
-    # Iterate the event signature parameters and see if the handler's signature satisfies each one
-    for index, (parameter_name, event_parameter) in enumerate(event_parameters.items()):
-        if event_parameter.kind == Parameter.POSITIONAL_ONLY:
-            if index > len(handler_positional_parameters) - 1:
-                if handler_positional_parameters[-1].kind != Parameter.VAR_POSITIONAL:
-                    raise TypeError(
-                        f"Missing required positional parameter: '{parameter_name}'"
-                    )
-
-            handler_parameter = handler_positional_parameters[index]
-            if handler_parameter != Parameter.VAR_POSITIONAL:
-                # Compare types
-                if handler_parameter.annotation != event_parameter.annotation:
-                    raise TypeError(
-                        f"Incorrect type annotation for positional parameter '{parameter_name}': expected {event_parameter.annotation}, but found {handler_parameter.annotation}"
-                    )
-
-                if (
-                    handler_parameter.return_annotation
-                    != event_parameter.return_annotation
-                ):
-                    raise TypeError(
-                        f"Incorrect return type annotation for positional parameter '{parameter_name}': expected {event_parameter.return_annotation}, but found {handler_parameter.return_annotation}"
-                    )
-
-        elif event_parameter.kind == Parameter.VAR_POSITIONAL:
-            # NOTE: This should never happen
-            raise TypeError(
-                "Invalid signature: events cannot declare variable positional arguments (e.g. *args)"
-            )
-
-        elif event_parameter.kind in [
-            Parameter.POSITIONAL_OR_KEYWORD,
-            Parameter.KEYWORD_ONLY,
-        ]:
-            if handler_parameter := handler_keyword_parameters.get(
-                parameter_name, None
-            ):
-                # We have the keyword arg, check the types
-                if handler_parameter.annotation != event_parameter.annotation:
-                    raise TypeError(
-                        f"Incorrect type annotation for parameter '{parameter_name}': expected {event_parameter.annotation}, but found {handler_parameter.annotation}"
-                    )
-            else:
-                # Check if the last parameter is a VAR_KEYWORD
-                if (
-                    list(handler_keyword_parameters.values())[-1].kind
-                    != Parameter.VAR_KEYWORD
-                ):
-                    raise TypeError(
-                        f"Missing required parameter: '{parameter_name}': expected signature: {event_signature}"
-                    )
-
-        else:
-            assert event_parameter.kind == Parameter.VAR_KEYWORD, event_parameter.kind
-
-
 # Context vars for asyncio tasks managed by run_event_handlers
-_event_context_var = ContextVar('servo.event', default=None)
-_connector_context_var = ContextVar('servo.connector', default=None)
-_connector_event_bus = WeakKeyDictionary()
+_event_context_var = contextvars.ContextVar("servo.event", default=None)
+_connector_context_var = contextvars.ContextVar("servo.connector", default=None)
+_connector_event_bus = weakref.WeakKeyDictionary()
 
 
 # NOTE: Boolean flag to know if we can safely reference base class from the metaclass
 _is_base_class_defined = False
 
-class Metaclass(ModelMetaclass):
+
+class Metaclass(pydantic.main.ModelMetaclass):
     def __new__(mcs, name, bases, namespace, **kwargs):
         # Decorate the class with an event registry, inheriting from our parent connectors
-        event_handlers: List[EventDescriptor] = []
+        event_handlers: List[EventHandler] = []
 
         for base in reversed(bases):
-            if (
-                _is_base_class_defined
-                and issubclass(base, Mixin)
-                and base is not Mixin
-            ):
+            if _is_base_class_defined and issubclass(base, Mixin) and base is not Mixin:
                 event_handlers.extend(base.__event_handlers__)
 
         new_namespace = {
@@ -695,11 +611,12 @@ class Mixin:
     def __init__(
         self,
         *args,
-        __connectors__: List[BaseConnector] = None,
+        __connectors__: List["servo.connector.BaseConnector"] = None,
         **kwargs,
     ):
         super().__init__(
-            *args, **kwargs,
+            *args,
+            **kwargs,
         )
 
         # NOTE: Connector references are held off the model so
@@ -756,7 +673,7 @@ class Mixin:
         return handler
 
     @property
-    def __connectors__(self) -> List[BaseConnector]:
+    def __connectors__(self) -> List["servo.connector.BaseConnector"]:
         return _connector_event_bus[self]
 
     def broadcast_event(
@@ -764,8 +681,8 @@ class Mixin:
         event: Union[Event, str],
         *args,
         first: bool = False,
-        include: Optional[List[BaseConnector]] = None,
-        exclude: Optional[List[BaseConnector]] = None,
+        include: Optional[List["servo.connector.BaseConnector"]] = None,
+        exclude: Optional[List["servo.connector.BaseConnector"]] = None,
         prepositions: Preposition = (
             Preposition.BEFORE | Preposition.ON | Preposition.AFTER
         ),
@@ -787,7 +704,7 @@ class Mixin:
                 exclude=exclude,
                 prepositions=prepositions,
                 return_exceptions=return_exceptions,
-                **kwargs
+                **kwargs,
             )
         )
 
@@ -796,8 +713,8 @@ class Mixin:
         event: Union[Event, str],
         *args,
         first: bool = False,
-        include: Optional[List[BaseConnector]] = None,
-        exclude: Optional[List[BaseConnector]] = None,
+        include: Optional[List["servo.connector.BaseConnector"]] = None,
+        exclude: Optional[List["servo.connector.BaseConnector"]] = None,
         prepositions: Preposition = (
             Preposition.BEFORE | Preposition.ON | Preposition.AFTER
         ),
@@ -831,7 +748,9 @@ class Mixin:
         if prepositions & Preposition.BEFORE:
             try:
                 for connector in connectors:
-                    await connector.run_event_handlers(event, Preposition.BEFORE, *args, **kwargs)
+                    await connector.run_event_handlers(
+                        event, Preposition.BEFORE
+                    )
             except CancelEventError as error:
                 # Cancelled by a before event handler. Unpack the result and return it
                 return [error.result]
@@ -841,23 +760,39 @@ class Mixin:
             if first:
                 # A single responder has been requested
                 for connector in connectors:
-                    results = await connector.run_event_handlers(event, Preposition.ON, *args, **kwargs)
+                    results = await connector.run_event_handlers(
+                        event, Preposition.ON, *args, **kwargs
+                    )
                     if results:
                         break
             else:
                 group = asyncio.gather(
-                    *list(map(lambda c: c.run_event_handlers(event, Preposition.ON, *args, **kwargs), connectors)),
+                    *list(
+                        map(
+                            lambda c: c.run_event_handlers(
+                                event, Preposition.ON, *args, **kwargs
+                            ),
+                            connectors,
+                        )
+                    ),
                     return_exceptions=return_exceptions,
                 )
                 results = await group
                 results = list(filter(lambda r: r is not None, results))
                 if results:
-                    results = reduce(lambda x, y: x+y, results)
+                    results = functools.reduce(lambda x, y: x + y, results)
 
         # Invoke the after event handlers
         if prepositions & Preposition.AFTER:
             await asyncio.gather(
-                *list(map(lambda c: c.run_event_handlers(event, Preposition.AFTER, results, *args, **kwargs), connectors))
+                *list(
+                    map(
+                        lambda c: c.run_event_handlers(
+                            event, Preposition.AFTER, results
+                        ),
+                        connectors,
+                    )
+                )
             )
 
         if first:
@@ -871,13 +806,15 @@ class Mixin:
         preposition: Preposition,
         *args,
         return_exceptions: bool = False,
-        **kwargs
+        **kwargs,
     ) -> Optional[List[EventResult]]:
         """
         Run handlers for the given event and preposition and return the results or None if there are no handlers.
         """
         if not isinstance(event, Event):
-            raise ValueError(f"event must be an Event object, got {event.__class__.__name__}")
+            raise ValueError(
+                f"event must be an Event object, got {event.__class__.__name__}"
+            )
 
         event_handlers = self.get_event_handlers(event, preposition)
         if not event_handlers:
@@ -886,7 +823,9 @@ class Mixin:
         results: List[EventResult] = []
         try:
             prev_connector_token = _connector_context_var.set(self)
-            prev_event_token = _event_context_var.set(EventContext(event=event, preposition=preposition))
+            prev_event_token = _event_context_var.set(
+                EventContext(event=event, preposition=preposition)
+            )
             for event_handler in event_handlers:
                 # NOTE: Explicit kwargs take precendence over those defined during handler declaration
                 handler_kwargs = event_handler.kwargs.copy()
@@ -894,7 +833,10 @@ class Mixin:
                 try:
                     async with event.on_handler_context_manager(self):
                         if asyncio.iscoroutinefunction(event_handler.handler):
-                            value = await asyncio.create_task(event_handler.handler(self, *args, **kwargs), name=f"{preposition}:{event}")
+                            value = await asyncio.create_task(
+                                event_handler.handler(self, *args, **kwargs),
+                                name=f"{preposition}:{event}",
+                            )
                         else:
                             value = event_handler.handler(self, *args, **kwargs)
 
@@ -951,4 +893,5 @@ class Mixin:
         """
         return _event_context_var.get()
 
+
 _is_base_class_defined = True
